import { __awaiter, __generator } from "tslib";
import { eip1559, eip1559_arbitrum, legacy, polygonGasStation } from "./adapters";
/**
 * Provide an estimate for the current gas price for a particular chain.
 * @param chainId The chain ID to query for gas prices.
 * @param provider A valid ethers provider.
 * @param legacyFallback In the case of an unrecognised chain, fall back to type 0 gas estimation.
 * @returns Am object of type GasPriceEstimate.
 */
export function getGasPriceEstimate(provider, chainId, legacyFallback) {
    if (legacyFallback === void 0) { legacyFallback = true; }
    return __awaiter(this, void 0, void 0, function () {
        var network, gasPriceFeeds, gasPriceFeed, gasPriceEstimate;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!!chainId) return [3 /*break*/, 2];
                    return [4 /*yield*/, provider.getNetwork()];
                case 1:
                    network = _a.sent();
                    chainId = network.chainId;
                    _a.label = 2;
                case 2:
                    gasPriceFeeds = {
                        1: eip1559,
                        10: eip1559,
                        137: polygonGasStation,
                        288: legacy,
                        324: legacy,
                        8453: eip1559,
                        42161: eip1559_arbitrum,
                    };
                    gasPriceFeed = gasPriceFeeds[chainId];
                    if (gasPriceFeed === undefined) {
                        if (!legacyFallback) {
                            throw new Error("No suitable gas price oracle for Chain ID ".concat(chainId));
                        }
                        gasPriceFeed = legacy;
                    }
                    return [4 /*yield*/, gasPriceFeed(provider, chainId)];
                case 3:
                    gasPriceEstimate = _a.sent();
                    return [2 /*return*/, gasPriceEstimate];
            }
        });
    });
}
//# sourceMappingURL=oracle.js.map