import { __awaiter, __generator } from "tslib";
import { utils } from "ethers";
/**
 * Checks if a contract is deployed at the given address
 * @param address The ETH address to check
 * @param provider A valid Ethers.js provider
 * @returns A boolean indicating if a contract is deployed at the given address or not (true = contract, false = no contract)
 */
export function isContractDeployedToAddress(address, provider) {
    return __awaiter(this, void 0, void 0, function () {
        var code;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    // A base case for if the address is null or malformed
                    if (!address || !utils.isAddress(address)) {
                        return [2 /*return*/, false];
                    }
                    return [4 /*yield*/, provider.getCode(address)];
                case 1:
                    code = _a.sent();
                    // If the code is not empty, then there is a contract at this address
                    return [2 /*return*/, code !== "0x"];
            }
        });
    });
}
//# sourceMappingURL=AddressUtils.js.map