import PinataClient from "@pinata/sdk";
/**
 * Build an IPFS client for interacting with the IPFS API
 * @param APIKey The project ID
 * @param secretAPIKey The project secret
 * @returns An IPFS client for interacting with Pinata
 */
export declare function buildIPFSClient(APIKey: string, secretAPIKey: string): PinataClient;
/**
 * Retrieves a value from an IPFS gateway
 * @param contentHash The content hash of the value to retrieve
 * @param publicGatewayURL The URL of the public IPFS gateway to use
 * @returns The value retrieved from the IPFS gateway
 * @throws Error if the value could not be retrieved
 */
export declare function retrieveValueFromIPFS(contentHash: string, publicGatewayURL: string): Promise<string>;
/**
 * Pins a value to the IPFS network
 * @param key A key to use for pinning the value. This is a metadata field.
 * @param content The value to pin
 * @param client The IPFS client to use
 * @returns The content hash of the pinned value
 */
export declare function storeValueInIPFS(key: string, content: string, client: PinataClient): Promise<string>;
//# sourceMappingURL=IPFSUtils.d.ts.map